@@ -10,9 +10,9 @@ RZ_IPI void rz_core_agraph_reset(RzCore *core) {
 }
 
 RZ_IPI void rz_core_agraph_add_node(RzCore *core, const char *title, const char *body) {
-	char *b = strdup(body);
+	char *b = rz_str_dup(body);
 	if (rz_str_startswith(b, "base64:")) {
-		char *newbody = strdup(b);
+		char *newbody = rz_str_dup(b);
 		if (!newbody) {
 			free(b);
 			return;
@@ -101,7 +101,7 @@ RZ_IPI void rz_core_agraph_print_interactive(RzCore *core) {
 }
 
 static void agraph_print_node_dot(RzANode *n, void *user) {
-	char *label = strdup(n->body);
+	char *label = rz_str_dup(n->body);
 	// label = rz_str_replace (label, "\n", "\\l", 1);
 
 	if (!label || !*label) {
@@ -243,7 +243,9 @@ RZ_IPI bool rz_core_agraph_add_shortcut(RzCore *core, RzAGraph *g, RzANode *an,
 	if (!shortcut) {
 		return false;
 	}
-	sdb_set(g->db, sdb_fmt("agraph.nodes.%s.shortcut", title), shortcut, 0);
+	char *key = rz_str_newf("agraph.nodes.%s.shortcut", title);
+	sdb_set(g->db, key, shortcut);
+	free(key);
 	// title + "[o{shortcut}]", so w + 3 ?
 	an->shortcut_w = strlen(shortcut) + 3;
 	free(shortcut);
@@ -266,7 +268,7 @@ RZ_IPI bool rz_core_agraph_apply(RzCore *core, RzGraph /*<RzGraphNodeInfo *>*/ *
 	if (!(core && core->graph && graph)) {
 		return false;
 	}
-	if (!create_agraph_from_graph_at(core->graph, graph, false)) {
+	if (!create_agraph_from_graph_at(core->graph, graph, false, rz_config_get_b(core->config, "scr.utf8"))) {
 		return false;
 	}
 	if (rz_core_agraph_is_shortcuts(core, core->graph)) {
