@@ -1,6 +1,9 @@
 // SPDX-FileCopyrightText: 2009-2020 pancake <pancake@nopcode.org>
 // SPDX-License-Identifier: LGPL-3.0-only
 
+#include <rz_cmd.h>
+#include <rz_util/rz_log.h>
+#include <rz_util/rz_num.h>
 #include <rz_util/rz_regex.h>
 #include <rz_core.h>
 #include <rz_debug.h>
@@ -25,25 +28,6 @@
 		} \
 	} while (0)
 
-static const char *help_msg_dcu[] = {
-	"Usage:", "dcu", " Continue until address",
-	"dcu.", "", "Alias for dcu $$ (continue until current address",
-	"dcu", " address", "Continue until address",
-	"dcu", " [..tail]", "Continue until the range",
-	"dcu", " [from] [to]", "Continue until the range",
-	NULL
-};
-
-static const char *help_msg_dmi[] = {
-	"Usage: dmi", "", " # List/Load Symbols",
-	"dmi", "[j|q|*] [libname] [symname]", "List symbols of target lib",
-	"dmia", "[j|q|*] [libname]", "List all info of target lib",
-	"dmi*", "", "List symbols of target lib in rizin commands",
-	"dmi.", "", "List closest symbol to the current address",
-	"dmiv", "", "Show address of given symbol for given lib",
-	NULL
-};
-
 struct dot_trace_ght {
 	RzGraph /*<struct trace_node *>*/ *graph;
 	Sdb *graphnodes;
@@ -56,7 +40,7 @@ struct trace_node {
 
 // XXX those tmp files are never removed and we shuoldnt use files for this
 static void set_profile_string(RzCore *core, const char *str) {
-	char *file = strdup(rz_config_get(core->config, "dbg.profile"));
+	char *file = rz_str_dup(rz_config_get(core->config, "dbg.profile"));
 	if (RZ_STR_ISEMPTY(file)) {
 		free(file);
 		file = rz_file_temp("rz-run");
@@ -72,7 +56,7 @@ static void cmd_debug_cont_syscall(RzCore *core, const char *_str) {
 	int i, *syscalls = NULL;
 	int count = 0;
 	if (_str && *_str) {
-		char *str = strdup(_str);
+		char *str = rz_str_dup(_str);
 		count = rz_str_word_set0(str);
 		syscalls = calloc(sizeof(int), count);
 		for (i = 0; i < count; i++) {
@@ -110,10 +94,10 @@ static RzGraphNode *get_graphtrace_node(RzGraph /*<struct trace_node *>*/ *g, Sd
 	char tn_key[TN_KEY_LEN];
 
 	snprintf(tn_key, TN_KEY_LEN, TN_KEY_FMT, tn->addr);
-	gn = (RzGraphNode *)(size_t)sdb_num_get(nodes, tn_key, NULL);
+	gn = (RzGraphNode *)(size_t)sdb_num_get(nodes, tn_key);
 	if (!gn) {
 		gn = rz_graph_add_node(g, tn);
-		sdb_num_set(nodes, tn_key, (ut64)(size_t)gn, 0);
+		sdb_num_set(nodes, tn_key, (ut64)(size_t)gn);
 	}
 	return gn;
 }
@@ -286,7 +270,6 @@ static bool step_until_inst(RzCore *core, const char *instr, bool regex) {
 		RZ_LOG_ERROR("wrong debugger state\n");
 		return false;
 	}
-	RzAsmOp asmop;
 	ut8 buf[32];
 	ut64 pc;
 	int ret;
@@ -311,20 +294,24 @@ static bool step_until_inst(RzCore *core, const char *instr, bool regex) {
 		rz_asm_set_pc(core->rasm, pc);
 		// TODO: speedup if instructions are in the same block as the previous
 		rz_io_read_at(core->io, pc, buf, sizeof(buf));
+		RzAsmOp asmop = { 0 };
 		ret = rz_asm_disassemble(core->rasm, &asmop, buf, sizeof(buf));
 		rz_cons_printf("0x%08" PFMT64x " %d %s\n", pc, ret, rz_asm_op_get_asm(&asmop)); // asmop.buf_asm);
 		if (ret > 0) {
 			const char *buf_asm = rz_asm_op_get_asm(&asmop);
 			if (regex) {
 				if (rz_regex_contains(instr, buf_asm, RZ_REGEX_ZERO_TERMINATED, RZ_REGEX_EXTENDED, RZ_REGEX_DEFAULT)) {
-					RZ_LOG_ERROR("core: esil: stop.\n");
+					rz_asm_op_fini(&asmop);
 					break;
 				}
 			} else {
 				if (strstr(buf_asm, instr)) {
+					rz_asm_op_fini(&asmop);
 					break;
 				}
 			}
+		} else {
+			rz_asm_op_fini(&asmop);
 		}
 	}
 	rz_core_reg_update_flags(core);
@@ -333,7 +320,7 @@ static bool step_until_inst(RzCore *core, const char *instr, bool regex) {
 }
 
 static bool step_until_optype(RzCore *core, RzList /*<char *>*/ *optypes_list) {
-	RzAnalysisOp op;
+	RzAnalysisOp op = { 0 };
 	ut8 buf[32];
 	ut64 pc;
 	int res = true;
@@ -383,7 +370,7 @@ static bool step_until_optype(RzCore *core, RzList /*<char *>*/ *optypes_list) {
 			pc = rz_reg_getv(core->analysis->reg, "PC");
 		}
 		rz_io_read_at(core->io, pc, buf, sizeof(buf));
-
+		rz_analysis_op_init(&op);
 		if (rz_analysis_op(core->dbg->analysis, &op, pc, buf, sizeof(buf), RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 			RZ_LOG_ERROR("rz_analysis_op failed\n");
 			res = false;
@@ -399,9 +386,11 @@ static bool step_until_optype(RzCore *core, RzList /*<char *>*/ *optypes_list) {
 				goto cleanup_after_push;
 			}
 		}
+		rz_analysis_op_fini(&op);
 	}
 
 cleanup_after_push:
+	rz_analysis_op_fini(&op);
 	rz_core_reg_update_flags(core);
 	rz_cons_break_pop();
 end:
@@ -489,7 +478,7 @@ static bool step_line(RzCore *core, int times) {
 }
 
 static void cmd_debug_backtrace(RzCore *core, ut64 len) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	ut64 addr;
 	if (!len) {
 		rz_bp_traptrace_list(core->dbg->bp);
@@ -517,8 +506,11 @@ static void cmd_debug_backtrace(RzCore *core, ut64 len) {
 			/* XXX Bottleneck..we need to reuse the bytes read by traptrace */
 			// XXX Do asm.arch should define the max size of opcode?
 			rz_io_read_at(core->io, addr, buf, 32); // XXX longer opcodes?
+			rz_analysis_op_fini(&aop);
+			rz_analysis_op_init(&aop);
 			rz_analysis_op(core->analysis, &aop, addr, buf, sizeof(buf), RZ_ANALYSIS_OP_MASK_BASIC);
 		} while (rz_bp_traptrace_at(core->dbg->bp, addr, aop.size));
+		rz_analysis_op_fini(&aop);
 		rz_bp_traptrace_enable(core->dbg->bp, false);
 	}
 }
@@ -558,7 +550,7 @@ static int dump_maps(RzCore *core, int perm, const char *filename) {
 			}
 			rz_io_read_at(core->io, map->addr, buf, map->size);
 			char *file = filename
-				? strdup(filename)
+				? rz_str_dup(filename)
 				: rz_str_newf("0x%08" PFMT64x "-0x%08" PFMT64x "-%s.dmp",
 					  map->addr, map->addr_end, rz_str_rwx_i(map->perm));
 			if (!rz_file_dump(file, buf, map->size, 0)) {
@@ -590,7 +582,7 @@ static void cmd_debug_current_modules(RzCore *core, RzOutputMode mode) { // "dmm
 		} else if (mode == RZ_OUTPUT_MODE_RIZIN) {
 			/* Escape backslashes (e.g. for Windows). */
 			char *escaped_path = rz_str_escape(map->file);
-			char *filtered_name = strdup(map->name);
+			char *filtered_name = rz_str_dup(map->name);
 			rz_name_filter(filtered_name, 0, true);
 			rz_cons_printf("f mod.%s @ 0x%08" PFMT64x "\n",
 				filtered_name, map->addr);
@@ -624,7 +616,7 @@ static void cmd_debug_modules(RzCore *core, RzCmdStateOutput *state) { // "dmm"
 		} else if (mode == RZ_OUTPUT_MODE_RIZIN) {
 			/* Escape backslashes (e.g. for Windows). */
 			char *escaped_path = rz_str_escape(map->file);
-			char *filtered_name = strdup(map->name);
+			char *filtered_name = rz_str_dup(map->name);
 			rz_name_filter(filtered_name, 0, true);
 			rz_cons_printf("f mod.%s @ 0x%08" PFMT64x "\n",
 				filtered_name, map->addr);
@@ -678,26 +670,17 @@ static RzDebugMap *get_closest_map(RzCore *core, ut64 addr) {
 	return NULL;
 }
 
-static RzOutputMode rad2mode(int mode) {
-	switch (mode) {
-	case RZ_MODE_PRINT:
-	default:
-		return RZ_OUTPUT_MODE_STANDARD;
-	case RZ_MODE_JSON:
-		return RZ_OUTPUT_MODE_JSON;
-	case RZ_MODE_SIMPLE:
-		return RZ_OUTPUT_MODE_QUIET;
-	case RZ_MODE_SIMPLEST:
-		return RZ_OUTPUT_MODE_QUIETEST;
-	case RZ_MODE_RIZINCMD:
-		return RZ_OUTPUT_MODE_RIZIN;
-	}
-}
-
-static bool get_bin_info(RzCore *core, const char *file, ut64 baseaddr, PJ *pj,
-	int mode, bool symbols_only, RzCoreBinFilter *filter) {
+/**
+ * \brief Hacky way to get the binary information of a file.
+ * It opens \p file into the current core->bin (backing up the previous pointer)
+ * and reads the information.
+ * Then closes it again and restores the old core-bin pointer.
+ */
+static bool get_bin_info(RzCore *core, const char *file, ut64 baseaddr,
+	RzCmdStateOutput *state, int action, RzCoreBinFilter *filter) {
 	int fd;
 	if ((fd = rz_io_fd_open(core->io, file, RZ_PERM_R, 0)) == -1) {
+		RZ_LOG_ERROR("Failed to open file: %s\n", file);
 		return false;
 	}
 	RzBinOptions opt = { 0 };
@@ -710,24 +693,11 @@ static bool get_bin_info(RzCore *core, const char *file, ut64 baseaddr, PJ *pj,
 	RzBinFile *obf = rz_bin_cur(core->bin);
 	RzBinFile *bf = rz_bin_open_io(core->bin, &opt);
 	if (!bf) {
+		RZ_LOG_ERROR("Failed to create RzBinFile for: %s\n", file);
 		rz_io_fd_close(core->io, fd);
 		return false;
 	}
-	int action = RZ_CORE_BIN_ACC_ALL & ~RZ_CORE_BIN_ACC_INFO;
-	if (symbols_only || filter->name) {
-		action = RZ_CORE_BIN_ACC_SYMBOLS;
-	} else if (mode == RZ_MODE_SET || mode == RZ_MODE_RIZINCMD) {
-		action &= ~RZ_CORE_BIN_ACC_ENTRIES & ~RZ_CORE_BIN_ACC_MAIN & ~RZ_CORE_BIN_ACC_MAPS;
-	}
-	if (mode == RZ_MODE_SET) {
-		rz_core_bin_apply_info(core, core->bin->cur, action);
-	} else {
-		RzCmdStateOutput state;
-		rz_cmd_state_output_init(&state, rad2mode(mode));
-		rz_core_bin_print(core, bf, action, filter, &state, NULL);
-		rz_cmd_state_output_print(&state);
-		rz_cmd_state_output_fini(&state);
-	}
+	rz_core_bin_print(core, bf, action, filter, state, NULL);
 	rz_bin_file_delete(core->bin, bf);
 	rz_bin_file_set_cur_binfile(core->bin, obf);
 	rz_io_fd_close(core->io, fd);
@@ -830,176 +800,114 @@ RZ_IPI RzCmdStatus rz_cmd_debug_dump_maps_writable_handler(RzCore *core, int arg
 	return RZ_CMD_STATUS_OK;
 }
 
-// dmi
-RZ_IPI int rz_cmd_debug_dmi(void *data, const char *input) {
-	RzCore *core = (RzCore *)data;
+static RzDebugMap *get_debug_map_from_lib_name(RzCore *core, const char *lib_name) {
+	ut64 addr = addroflib(core, rz_file_basename(lib_name));
+	if (addr == UT64_MAX) {
+		RZ_LOG_ERROR("Unknown library '%s' not found\n", lib_name);
+		return NULL;
+	}
+
+	RzDebugMap *map = get_closest_map(core, addr);
+	if (!map) {
+		RZ_LOG_ERROR("Didn't find library map at 0x%" PFMT64x "\n", addr);
+		return NULL;
+	}
+	return map;
+}
+
+RZ_IPI RzCmdStatus rz_cmd_debug_dmi_handler(RzCore *core, int argc, const char **argv, RzCmdStateOutput *state) {
 	CMD_CHECK_DEBUG_DEAD(core);
-	RzDebugMap *map;
-	ut64 addr = core->offset;
-	switch (input[0]) {
-	case '\0': // "dmi" alias of "dmm"
-	{
-		RzCmdStateOutput state = { 0 };
-		rz_cmd_state_output_init(&state, RZ_OUTPUT_MODE_STANDARD);
-		cmd_debug_modules(core, &state);
-		rz_cmd_state_output_print(&state);
-		rz_cmd_state_output_fini(&state);
+	if (argc == 1) {
+		// Effectively an alias for 'dmm'
+		cmd_debug_modules(core, state);
+		rz_cmd_state_output_print(state);
 		rz_cons_flush();
-		break;
+		return RZ_CMD_STATUS_OK;
 	}
-	case ' ': // "dmi "
-	case '*': // "dmi*"
-	case 'v': // "dmiv"
-	case 'j': // "dmij"
-	case 'q': // "dmiq"
-	case 'a': // "dmia"
-	{
-		const char *libname = NULL, *symname = NULL, *a0;
-		int mode;
-		ut64 baddr = 0LL;
-		char *ptr;
-		int i = 1;
-		bool symbols_only = true;
-		if (input[0] == 'a') {
-			symbols_only = false;
-			input++;
-		}
-		PJ *pj = NULL;
-		switch (input[0]) {
-		case 's':
-			mode = RZ_MODE_SET;
-			break;
-		case '*':
-			mode = RZ_MODE_RIZINCMD;
-			break;
-		case 'j':
-			mode = RZ_MODE_JSON;
-			pj = pj_new();
-			if (!pj) {
-				return false;
-			}
-			break;
-		case 'q':
-			mode = input[1] == 'q' ? input++, RZ_MODE_SIMPLEST : RZ_MODE_SIMPLE;
-			break;
-		default:
-			mode = RZ_MODE_PRINT;
-			break;
-		}
-		ptr = strdup(input[0] ? rz_str_trim_head_ro(input + 1) : "");
-		if (!ptr || !*ptr) {
-			rz_core_cmd(core, "dmm", 0);
-			free(ptr);
-			pj_free(pj);
-			break;
-		}
-		if (symbols_only) {
-			i = rz_str_word_set0(ptr);
-		}
-		switch (i) {
-		case 2:
-			symname = rz_str_word_get0(ptr, 1);
-			// fall through
-		case 1:
-			a0 = rz_str_word_get0(ptr, 0);
-			addr = rz_num_get(core->num, a0);
-			if (!addr || addr == UT64_MAX) {
-				libname = rz_str_word_get0(ptr, 0);
-			}
-			break;
-		}
-		if (libname && !addr) {
-			addr = addroflib(core, rz_file_basename(libname));
-			if (addr == UT64_MAX) {
-				RZ_LOG_ERROR("core: Unknown library, or not found in dm\n");
-			}
-		}
-		map = get_closest_map(core, addr);
-		if (map) {
-			RzCoreBinFilter filter;
-			filter.offset = UT64_MAX;
-			filter.name = (char *)symname;
-			baddr = map->addr;
 
-			if (libname) {
-				const char *file = map->file ? map->file : map->name;
-				char *newfile = NULL;
-				if (!rz_file_exists(file)) {
-					newfile = rz_file_temp("memlib");
-					if (newfile) {
-						file = newfile;
-						rz_core_dump(core, file, baddr, map->size, false);
-					}
-				}
-				get_bin_info(core, file, baddr, pj, mode, symbols_only, &filter);
-				if (newfile) {
-					if (!rz_file_rm(newfile)) {
-						RZ_LOG_ERROR("core: Error when removing %s\n", newfile);
-					}
-					free(newfile);
-				}
-			} else {
-				RzBinFile *bf = rz_bin_cur(core->bin);
-				if (bf) {
-					rz_bin_set_baddr(core->bin, map->addr);
-					RzCmdStateOutput state;
-					rz_cmd_state_output_init(&state, rad2mode(mode));
-					rz_core_bin_print(core, bf, RZ_CORE_BIN_ACC_SYMBOLS, &filter, &state, NULL);
-					rz_cmd_state_output_print(&state);
-					rz_cmd_state_output_fini(&state);
-					rz_bin_set_baddr(core->bin, baddr);
-				}
-			}
-		}
-		if (mode == RZ_MODE_JSON) {
-			rz_cons_println(pj_string(pj));
-			pj_free(pj);
-		}
-		free(ptr);
-	} break;
-	case '.': // "dmi."
-	{
-		map = get_closest_map(core, addr);
-		if (map) {
-			ut64 closest_addr = UT64_MAX;
-			RzBinObject *o = rz_bin_cur_object(core->bin);
-			RzPVector *symbols = o ? (RzPVector *)rz_bin_object_get_symbols(o) : NULL;
-			RzBinSymbol *symbol, *closest_symbol = NULL;
-			void **iter;
+	const char *lib_name = argc >= 2 ? argv[1] : NULL;
+	const char *sym_name = argc == 3 ? argv[2] : NULL;
 
-			rz_pvector_foreach (symbols, iter) {
-				symbol = *iter;
-				if (symbol->vaddr > addr) {
-					if (symbol->vaddr - addr < closest_addr) {
-						closest_addr = symbol->vaddr - addr;
-						closest_symbol = symbol;
-					}
-				} else {
-					if (addr - symbol->vaddr < closest_addr) {
-						closest_addr = addr - symbol->vaddr;
-						closest_symbol = symbol;
-					}
-				}
-			}
-			RzBinFile *bf = rz_bin_cur(core->bin);
-			if (closest_symbol && bf) {
-				RzCoreBinFilter filter;
-				filter.offset = UT64_MAX;
-				filter.name = (char *)closest_symbol->name;
+	RzCoreBinFilter filter = { .offset = UT64_MAX, .name = sym_name };
+	int action = RZ_CORE_BIN_ACC_SYMBOLS;
 
-				rz_bin_set_baddr(core->bin, map->addr);
-				RzCmdStateOutput state;
-				rz_cmd_state_output_init(&state, RZ_OUTPUT_MODE_STANDARD);
-				rz_core_bin_print(core, bf, RZ_CORE_BIN_ACC_SYMBOLS, &filter, &state, NULL);
-				rz_cmd_state_output_print(&state);
-				rz_cmd_state_output_fini(&state);
-			}
-		}
-	} break;
-	default:
-		rz_core_cmd_help(core, help_msg_dmi);
-		break;
+	RzDebugMap *map = get_debug_map_from_lib_name(core, lib_name);
+	if (!map) {
+		RZ_LOG_ERROR("Failed to get map from %s\n", lib_name);
+		return RZ_CMD_STATUS_ERROR;
 	}
+	const char *file = map->file ? map->file : map->name;
+	if (!get_bin_info(core, file, map->addr, state, action, &filter)) {
+		RZ_LOG_ERROR("Failed to get binary information for map: '%s' in file: '%s'\n", map->name, file);
+		return RZ_CMD_STATUS_ERROR;
+	}
+	return RZ_CMD_STATUS_OK;
+}
+
+RZ_IPI RzCmdStatus rz_cmd_debug_dmi_all_handler(RzCore *core, int argc, const char **argv, RzCmdStateOutput *state) {
+	CMD_CHECK_DEBUG_DEAD(core);
+	if (argc == 1) {
+		cmd_debug_modules(core, state);
+		rz_cmd_state_output_print(state);
+		rz_cons_flush();
+		return RZ_CMD_STATUS_OK;
+	}
+	const char *lib_name = argv[1];
+	RzDebugMap *map = get_debug_map_from_lib_name(core, lib_name);
+	if (!map) {
+		RZ_LOG_ERROR("Failed to get map from %s\n", lib_name);
+		return RZ_CMD_STATUS_ERROR;
+	}
+	const char *file = map->file ? map->file : map->name;
+	RzCoreBinFilter filter = { .offset = UT64_MAX, .name = NULL };
+	int action = RZ_CORE_BIN_ACC_ALL & ~RZ_CORE_BIN_ACC_INFO;
+	if (!get_bin_info(core, file, map->addr, state, action, &filter)) {
+		RZ_LOG_ERROR("Failed to get binary information for map: '%s' in file: '%s'\n", map->name, file);
+		return RZ_CMD_STATUS_ERROR;
+	}
+	return RZ_CMD_STATUS_OK;
+}
+
+RZ_IPI RzCmdStatus rz_cmd_debug_dmi_closest_handler(RzCore *core, int argc, const char **argv, RzCmdStateOutput *state) {
+	RzBinObject *obj = rz_bin_cur_object(core->bin);
+	if (!obj) {
+		RZ_LOG_ERROR("No object present.\n");
+		return RZ_CMD_STATUS_ERROR;
+	}
+	const RzPVector *symbols = rz_bin_object_get_symbols(obj);
+	if (!symbols) {
+		RZ_LOG_ERROR("Failed to get symbols from object.\n");
+		return RZ_CMD_STATUS_ERROR;
+	}
+	ut64 addr = core->offset;
+	ut64 closest_addr = UT64_MAX;
+	RzBinSymbol *symbol, *closest_symbol = NULL;
+	void **iter;
+	rz_pvector_foreach (symbols, iter) {
+		symbol = *iter;
+		if (symbol->vaddr > addr) {
+			if (symbol->vaddr - addr < closest_addr) {
+				closest_addr = symbol->vaddr - addr;
+				closest_symbol = symbol;
+			}
+		} else {
+			if (addr - symbol->vaddr < closest_addr) {
+				closest_addr = addr - symbol->vaddr;
+				closest_symbol = symbol;
+			}
+		}
+	}
+	if (!closest_symbol) {
+		RZ_LOG_ERROR("Did not found any symbol close to 0x%" PFMT64x "\n", addr);
+		return RZ_CMD_STATUS_ERROR;
+	}
+	RzBinFile *bf = rz_bin_cur(core->bin);
+	if (!bf) {
+		RZ_LOG_ERROR("Failed to get current binary file.\n");
+		return RZ_CMD_STATUS_ERROR;
+	}
+	RzCoreBinFilter filter = { .offset = UT64_MAX, .name = closest_symbol->name };
+	rz_core_bin_print(core, bf, RZ_CORE_BIN_ACC_SYMBOLS, &filter, state, NULL);
 	return RZ_CMD_STATUS_OK;
 }
 
@@ -1131,20 +1039,35 @@ RZ_IPI RzCmdStatus rz_cmd_debug_dmL_handler(RzCore *core, int argc, const char *
 	return RZ_CMD_STATUS_OK;
 }
 
-// dmx
-RZ_IPI int rz_cmd_debug_heap_jemalloc(void *data, const char *input) {
-	RzCore *core = (RzCore *)data;
+static RzCmdStatus call_map_jemalloc(RzCore *core, char type, const char *arg) {
 	CMD_CHECK_DEBUG_DEAD(core);
 #if HAVE_JEMALLOC
 	if (core->rasm->bits == 64) {
-		return cmd_dbg_map_jemalloc_64(core, input);
+		cmd_dbg_map_jemalloc_64(core, type, arg);
 	} else {
-		return cmd_dbg_map_jemalloc_32(core, input);
+		cmd_dbg_map_jemalloc_32(core, type, arg);
 	}
+	return RZ_CMD_STATUS_OK;
 #endif
+	RZ_LOG_ERROR("JEMALLOC not supported.\n");
 	return RZ_CMD_STATUS_ERROR;
 }
 
+// "dmxa"
+RZ_IPI RzCmdStatus rz_cmd_debug_heap_jemalloc_a_handler(RzCore *core, int argc, const char **argv) {
+	return call_map_jemalloc(core, 'a', argc == 1 ? "" : argv[1]);
+}
+
+// "dmxb"
+RZ_IPI RzCmdStatus rz_cmd_debug_heap_jemalloc_b_handler(RzCore *core, int argc, const char **argv) {
+	return call_map_jemalloc(core, 'b', argc == 1 ? "" : argv[1]);
+}
+
+// "dmxc"
+RZ_IPI RzCmdStatus rz_cmd_debug_heap_jemalloc_c_handler(RzCore *core, int argc, const char **argv) {
+	return call_map_jemalloc(core, 'c', argv[1]);
+}
+
 static void backtrace_vars(RzCore *core, RzList /*<RzDebugFrame *>*/ *frames) {
 	RzDebugFrame *f;
 	RzListIter *iter;
@@ -1294,7 +1217,7 @@ RZ_IPI void rz_core_debug_bp_add(RzCore *core, ut64 addr, const char *arg_perm,
 			rz_bp_item_set_name(bpi, name);
 			free(name);
 		} else {
-			bpi->name = strdup(f->name);
+			bpi->name = rz_str_dup(f->name);
 		}
 	} else {
 		char *name = rz_str_newf("0x%08" PFMT64x, addr);
@@ -1349,8 +1272,7 @@ static void trace_traverse(RTree *t) {
 }
 
 static void do_debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_addr) {
-	bool trace_libs = rz_config_get_i(core->config, "dbg.trace.libs");
-	bool shallow_trace = rz_config_get_i(core->config, "dbg.trace.inrange");
+	bool shallow_trace = rz_config_get_b(core->config, "dbg.trace.inrange");
 	HtUP *tracenodes = core->dbg->tracenodes;
 	RTree *tr = core->dbg->tree;
 	RzDebug *dbg = core->dbg;
@@ -1359,20 +1281,13 @@ static void do_debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_ad
 	ut64 addr = 0;
 	int n = 0;
 
-	if (!trace_libs) {
-#if NOOP
-		RzList *bounds = rz_core_get_boundaries_prot(core, -1, "dbg.program", "search");
-		rz_list_free(bounds);
-#endif
-	}
-
 	/* set root if not already present */
 	rz_tree_add_node(tr, NULL, NULL);
 	cur = tr->root;
 
 	while (true) {
 		ut8 buf[32];
-		RzAnalysisOp aop;
+		RzAnalysisOp aop = { 0 };
 		int addr_in_range;
 
 		if (rz_cons_is_breaked()) {
@@ -1399,6 +1314,7 @@ static void do_debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_ad
 		addr_in_range = addr >= from && addr < to;
 
 		rz_io_read_at(core->io, addr, buf, sizeof(buf));
+		rz_analysis_op_init(&aop);
 		rz_analysis_op(core->analysis, &aop, addr, buf, sizeof(buf), RZ_ANALYSIS_OP_MASK_BASIC);
 		eprintf("%d %" PFMT64x "\r", n++, addr);
 		switch (aop.type) {
@@ -1446,6 +1362,7 @@ static void do_debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_ad
 			}
 			break;
 		}
+		rz_analysis_op_fini(&aop);
 	}
 }
 
@@ -1454,7 +1371,7 @@ static void debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_addr)
 	int t = core->dbg->trace->enabled;
 
 	if (rz_debug_is_dead(core->dbg)) {
-		RZ_LOG_ERROR("core: No process to debug.");
+		RZ_LOG_ERROR("core: No process to debug.\n");
 		return;
 	}
 	core->dbg->trace->enabled = 0;
@@ -1477,69 +1394,6 @@ static void debug_trace_calls(RzCore *core, ut64 from, ut64 to, ut64 final_addr)
 	rz_cons_break_pop();
 }
 
-static bool cmd_dcu(RzCore *core, const char *input) {
-	const char *ptr = NULL;
-	ut64 from, to, pc;
-	bool dcu_range = false;
-	bool invalid = (!input[0] || !input[1] || !input[2]);
-	if (invalid || (input[2] != ' ' && input[2] != '.')) {
-		rz_core_cmd_help(core, help_msg_dcu);
-		return false;
-	}
-	to = UT64_MAX;
-	if (input[2] == '.') {
-		ptr = strchr(input + 3, ' ');
-		if (ptr) { // TODO: put '\0' in *ptr to avoid
-			from = rz_num_tail(core->num, core->offset, input + 2);
-			if (ptr[1] == '.') {
-				to = rz_num_tail(core->num, core->offset, ptr + 2);
-			} else {
-				to = rz_num_math(core->num, ptr + 1);
-			}
-			dcu_range = true;
-		} else {
-			from = rz_num_tail(core->num, core->offset, input + 2);
-		}
-	} else {
-		ptr = strchr(input + 3, ' ');
-		if (ptr) { // TODO: put '\0' in *ptr to avoid
-			from = rz_num_math(core->num, input + 3);
-			if (ptr[1] == '.') {
-				to = rz_num_tail(core->num, core->offset, ptr + 2);
-			} else {
-				to = rz_num_math(core->num, ptr + 1);
-			}
-			dcu_range = true;
-		} else {
-			from = rz_num_math(core->num, input + 3);
-		}
-	}
-	if (core->num->nc.errors && rz_cons_is_interactive()) {
-		RZ_LOG_ERROR("core: Cannot continue until unknown address '%s'\n", core->num->nc.calc_buf);
-		return false;
-	}
-	if (to == UT64_MAX) {
-		to = from;
-	}
-	if (dcu_range) {
-		rz_cons_break_push(NULL, NULL);
-		do {
-			if (rz_cons_is_breaked()) {
-				break;
-			}
-			rz_debug_step(core->dbg, 1);
-			rz_debug_reg_sync(core->dbg, RZ_REG_TYPE_GPR, false);
-			pc = rz_debug_reg_get(core->dbg, "PC");
-			RZ_LOG_WARN("core: Continue 0x%08" PFMT64x " > 0x%08" PFMT64x " < 0x%08" PFMT64x "\n",
-				from, pc, to);
-		} while (pc < from || pc > to);
-		rz_cons_break_pop();
-	} else {
-		return rz_core_debug_continue_until(core, from, to);
-	}
-	return true;
-}
-
 // dsu
 RZ_IPI RzCmdStatus rz_cmd_debug_step_until_handler(RzCore *core, int argc, const char **argv) {
 	rz_reg_arena_swap(core->dbg->reg, true);
@@ -1657,13 +1511,6 @@ RZ_IPI RzCmdStatus rz_cmd_debug_traces_reset_handler(RzCore *core, int argc, con
 	return RZ_CMD_STATUS_OK;
 }
 
-// dta
-RZ_IPI int rz_cmd_debug_trace_addr(void *data, const char *input) {
-	RzCore *core = (RzCore *)data;
-	rz_debug_trace_at(core->dbg, input);
-	return 0;
-}
-
 // dtc
 RZ_IPI RzCmdStatus rz_cmd_debug_trace_calls_handler(RzCore *core, int argc, const char **argv) {
 	ut64 from = argc > 1 ? rz_num_math(core->num, argv[1]) : 0;
@@ -2135,7 +1982,7 @@ RZ_IPI RzCmdStatus rz_cmd_debug_toggle_bp_trace_index_handler(RzCore *core, int
 // dbh
 RZ_IPI RzCmdStatus rz_cmd_debug_bp_plugin_handler(RzCore *core, int argc, const char **argv) {
 	if (argc == 1) {
-		rz_bp_plugin_list(core->dbg->bp);
+		rz_bp_plugin_print(core->dbg->bp);
 	} else if (argc == 2) {
 		if (!rz_bp_use(core->dbg->bp, argv[1])) {
 			RZ_LOG_ERROR("Failed to set breakpoint plugin handler to %s\n", argv[1]);
@@ -2173,12 +2020,15 @@ RZ_IPI RzCmdStatus rz_cmd_debug_display_bt_handler(RzCore *core, int argc, const
 	RzBacktrace *bt;
 	PJ *pj = state->d.pj;
 	rz_cmd_state_output_array_start(state);
+	rz_cmd_state_output_set_columnsf(state, "dxxdss", "idx", "pc", "sp", "frame_size", "fname", "desc");
 	rz_list_foreach (list, iter, bt) {
 		switch (mode) {
 		case RZ_OUTPUT_MODE_STANDARD: {
 			rz_cons_printf("%d  %s sp: %s  %-5d"
-				       "[%s]  %s %s\n",
-				i++, bt->pcstr, bt->spstr, bt->frame->size, bt->fcn ? bt->fcn->name : "??", rz_str_get(bt->flagdesc), rz_str_get(bt->flagdesc2));
+				       "[%s]%s%s%s%s\n",
+				i++, bt->pcstr, bt->spstr, bt->frame->size, bt->fcn ? bt->fcn->name : "??",
+				bt->flagdesc ? "  " : "", rz_str_get(bt->flagdesc),
+				bt->flagdesc2 ? " " : "", rz_str_get(bt->flagdesc2));
 			break;
 		}
 		case RZ_OUTPUT_MODE_RIZIN: {
@@ -2199,6 +2049,12 @@ RZ_IPI RzCmdStatus rz_cmd_debug_display_bt_handler(RzCore *core, int argc, const
 			i++;
 			break;
 		}
+		case RZ_OUTPUT_MODE_TABLE: {
+			rz_table_add_rowf(state->d.t, "dxxdss", i, bt->frame->addr, bt->frame->sp, bt->frame->size,
+				bt->fcn ? bt->fcn->name : "", bt->desc);
+			i++;
+			break;
+		}
 		case RZ_OUTPUT_MODE_QUIET: {
 			rz_cons_printf("%s\n", bt->pcstr);
 			break;
@@ -2404,6 +2260,19 @@ RZ_IPI RzCmdStatus rz_cmd_debug_continue_call_handler(RzCore *core, int argc, co
 	return RZ_CMD_STATUS_OK;
 }
 
+// dcco
+RZ_IPI RzCmdStatus rz_cmd_debug_continue_call_over_handler(RzCore *core, int argc, const char **argv) {
+	CMD_CHECK_DEBUG_DEAD(core);
+	rz_cons_break_push(rz_core_static_debug_stop, core->dbg);
+	rz_reg_arena_swap(core->dbg->reg, true);
+
+	rz_debug_continue_until_optype(core->dbg, RZ_ANALYSIS_OP_TYPE_CALL, 1);
+
+	rz_cons_break_pop();
+	rz_core_dbg_follow_seek_register(core);
+	return RZ_CMD_STATUS_OK;
+}
+
 // dccu
 RZ_IPI RzCmdStatus rz_cmd_debug_continue_unknown_call_handler(RzCore *core, int argc, const char **argv) {
 	CMD_CHECK_DEBUG_DEAD(core);
@@ -2538,22 +2407,15 @@ RZ_IPI RzCmdStatus rz_cmd_debug_continue_syscall_handler(RzCore *core, int argc,
 }
 
 // dcu
-RZ_IPI int rz_cmd_debug_continue_until(void *data, const char *input) {
-	RzCore *core = (RzCore *)data;
+RZ_IPI RzCmdStatus rz_cmd_debug_continue_until_handler(RzCore *core, int argc, const char **argv) {
 	CMD_CHECK_DEBUG_DEAD(core);
 	rz_cons_break_push(rz_core_static_debug_stop, core->dbg);
-	if (input[0] == '?') {
-		rz_core_cmd_help(core, help_msg_dcu);
-	} else if (input[0] == '.' || input[0] == '\0') {
-		cmd_dcu(core, "cu $$");
-	} else {
-		char *tmpinp = rz_str_newf("cu %s", input + 1);
-		cmd_dcu(core, tmpinp);
-		free(tmpinp);
-	}
+	ut64 addr = rz_num_math(core->num, argv[1]);
+	RZ_LOG_INFO("Continue until 0x%" PFMT64x "\n", addr);
+	bool success = rz_core_debug_continue_until(core, addr);
 	rz_cons_break_pop();
 	rz_core_dbg_follow_seek_register(core);
-	return RZ_CMD_STATUS_OK;
+	return success ? RZ_CMD_STATUS_OK : RZ_CMD_STATUS_ERROR;
 }
 
 RZ_IPI RzCmdStatus rz_cmd_debug_handler_list_handler(RzCore *core, int argc, const char **argv, RzCmdStateOutput *state) {
@@ -2692,16 +2554,19 @@ RZ_IPI RzCmdStatus rz_cmd_debug_step_prog_handler(RzCore *core, int argc, const
 		rz_debug_reg_sync(core->dbg, RZ_REG_TYPE_GPR, false);
 		ut64 addr = rz_debug_reg_get(core->dbg, "PC");
 		rz_io_read_at(core->io, addr, buf, sizeof(buf));
-		RzAnalysisOp aop;
+		RzAnalysisOp aop = { 0 };
+		rz_analysis_op_init(&aop);
 		rz_analysis_op(core->analysis, &aop, addr, buf, sizeof(buf), RZ_ANALYSIS_OP_MASK_BASIC);
 		if (aop.type == RZ_ANALYSIS_OP_TYPE_CALL) {
 			RzBinObject *o = rz_bin_cur_object(core->bin);
 			RzBinSection *s = rz_bin_get_section_at(o, aop.jump, true);
 			if (!s) {
 				rz_debug_step_over(core->dbg, times);
+				rz_analysis_op_fini(&aop);
 				continue;
 			}
 		}
+		rz_analysis_op_fini(&aop);
 		rz_debug_step(core->dbg, 1);
 	}
 	rz_core_reg_update_flags(core);
@@ -3106,7 +2971,7 @@ RZ_IPI RzCmdStatus rz_cmd_debug_core_dump_generate_handler(RzCore *core, int arg
 		RZ_LOG_ERROR("core: Not debugging, can't write core file.\n");
 		return RZ_CMD_STATUS_ERROR;
 	}
-	char *corefile = argc > 1 ? strdup(argv[1]) : rz_str_newf("core.%u", core->dbg->pid);
+	char *corefile = argc > 1 ? rz_str_dup(argv[1]) : rz_str_newf("core.%u", core->dbg->pid);
 	RZ_LOG_WARN("core: Writing to file '%s'\n", corefile);
 	rz_file_rm(corefile);
 	RzBuffer *dst = rz_buf_new_file(corefile, O_RDWR | O_CREAT, 0644);
