@@ -166,7 +166,7 @@ RZ_API RZ_OWN RzList /*<RzCompareData *>*/ *rz_core_cmp_disasm(RzCore *core, ut6
 		goto error_goto;
 	}
 	cmp_list->free = (RzListFree)&rz_core_cmp_free;
-	RzAsmOp op, op2;
+	RzAsmOp op = { 0 }, op2 = { 0 };
 	int i, j;
 	ut8 *buf = calloc(len + 32, 1);
 	if (!buf) {
@@ -192,9 +192,9 @@ RZ_API RZ_OWN RzList /*<RzCompareData *>*/ *rz_core_cmp_disasm(RzCore *core, ut6
 			buf + j, len - j);
 
 		comp->len = UT8_MAX;
-		comp->data1 = (ut8 *)strdup(rz_strbuf_get(&op.buf_asm));
+		comp->data1 = (ut8 *)rz_str_dup(rz_strbuf_get(&op.buf_asm));
 		comp->addr1 = addr1 + i;
-		comp->data2 = (ut8 *)strdup(rz_strbuf_get(&op2.buf_asm));
+		comp->data2 = (ut8 *)rz_str_dup(rz_strbuf_get(&op2.buf_asm));
 		comp->addr2 = addr2 + j;
 		comp->same = !strcmp((char *)comp->data1, (char *)comp->data2); // we can assume that instructions can be represented as plain chars
 		rz_list_append(cmp_list, comp);
@@ -207,6 +207,8 @@ RZ_API RZ_OWN RzList /*<RzCompareData *>*/ *rz_core_cmp_disasm(RzCore *core, ut6
 			op2.size = 1;
 		}
 		j += op2.size;
+		rz_asm_op_fini(&op);
+		rz_asm_op_fini(&op2);
 	}
 
 	free(buf);
