@@ -1,2 +1,3 @@
 # CS_ARCH_AARCH64, 0, None
-0x9f,0x3c,0x03,0xd5 = dfb
+
+0x9f,0x3c,0x03,0xd5 == dfb
